package com.cm6123.monopoly.game;

import com.cm6123.monopoly.dice.Dice;

import java.util.Scanner;

/**
 * class for game actions if a player lands on a tile that is useful.
 */
public class GameAction implements GameHandler {

    /**
     * The board where the properties and train stations lie on.
     * (I cant fix this checkstyle error [MissingJavadocComment]) ??
     */
    private final Board board;

    /**
     * Constructor for the gameboard.
     *
     * @param gameboard the game board used for game actions
     */
    public GameAction(final Board gameboard) {
        this.board = gameboard;
    }

    /**
     * Performs game actions when a player lands on a property.
     * This method prompts the player to buy the property if possible, and handles the player's choice.
     *
     * @param player the player who landed on the property
     */
    @Override
    public void landedOnProperty(final Player player) {
        int position = player.getPosition();

        // Check if the property at the current position can be bought
        if (this.board.canBuyProperty(position)) {
            String propertyName = this.board.getGameBoard(position);

            Scanner scanner = new Scanner(System.in);

            // incased in a while loop so the player has to type either Y or N
            while (true) {
                System.out.println("Would you like to buy " + propertyName + " for £200? (Y/N)");
                String input = scanner.next().toUpperCase();

                // if the user input is Y subtract 200 from the players account and make thm th woner of the property
                if (input.equals("Y")) {
                    if (player.getMoney() >= 200) {
                        this.board.buyProperty(position, player);
                    } else {
                        System.out.println("You do not have enough money to buy " + propertyName + ". Moving on...");
                    }
                    break;
                    // if user types N then user doesn't want to buy the property and then continue the game
                } else if (input.equals("N")) {
                    System.out.println("You have chosen N, moving on...");
                    break;
                } else { // force the player to enter either Y or N if they typed anything else
                    System.out.println("Invalid input. Please enter (Y/N)");
                }
            }
        }
    }


    /**
     * Performs game actions when a player lands on a train station.
     * Allows the player to roll again to determine a ticket cost until they enter 'R'.
     *
     * @param player the player landing on the train station
     */
    @Override
    public void landedOnTrainStation(final Player player) {
        Movement movement = new Movement();
        Dice dice = new Dice(6);

        // Roll two dice to determine the result
        int roll1 = dice.roll();
        int roll2 = dice.roll();
        int diceResult = roll1 + roll2;

        Scanner scanner = new Scanner(System.in);

        // Loop until the user enters 'R'
        while (true) {
            // Make the user roll again to get the ticket amount they nede to pay
            System.out.println("Enter 'R' to Roll again to determine your ticket");

            String input = scanner.next().toUpperCase();

            if (input.equals("R")) {
                System.out.println("You rolled " + diceResult);

                // Calculate ticket cost based on dice result
                int trainTicket = 10;
                int ticket = trainTicket * diceResult;

                if (player.getMoney() > 0) {
                    // Subtract ticket cost from player's money and display updated balance
                    System.out.println("You pay £" + ticket + " to the banker");
                    player.subtractMoney(ticket);
                    System.out.println("You now have £" + player.getMoney() + " in your bank account");

                    // Pay the ticket cost to the banker
                    Banker banker = new Banker();
                    banker.getMoneyFromPlayer(ticket);

                    // Exit the loop since the user has entered 'R'
                    break;
                }
            } else {
                // Make the user enter 'R' or game doesn't continue
                System.out.println("Invalid input. Please enter 'R'");
            }
        }
    }
}
