package com.cm6123.monopoly.game;

import java.util.ArrayList;

/**
 * Class for the player which holds the names, money and position.
 */
public class Player {

    /**
     * The name of the player.
     */
    private String name;

    /**
     * The amount of money the player has.
     */
    private int money;

    /**
     * The position of the player on the game board.
     */
    private int position;

    /**
     * Checking if the player has bought properties, initially false.
     */
    private static boolean hasAssets = false;



    /**
     * Constructor for player with the user inputed name, initial money, and starting position.
     *
     * @param playerName The name of the player.
     * @param playerMoney The initial amount of money the player has.
     * @param playerPosition The starting position of the player on the game board.
     */
    public Player(final String playerName, final int playerMoney,  final int playerPosition) {
        this.name = playerName;
        this.money = playerMoney;
        this.position = playerPosition;
    }

    /**
     *
     * @param amount amount of money added to the players balance.
     */
    public void addMoney(final int amount) {
        this.money += amount;
    }

    /**
     *
     * @param amount amount of money subtracted from the players balance.
     */
    public void subtractMoney(final int amount) {
        this.money -= amount;
    }

    /**
     * Gets the name of the player.
     *
     * @return The name of the player.
     */
    public String getName() {
        return name;
    }

    /**
     * Sets the name of the player.
     *
     * @param playerName
     */
    public void setName(final String playerName) {
        this.name = playerName;
    }

    /**
     * Gets the amount of money the player has.
     *
     * @return The amount of money the player has.
     */
    public int getMoney() {
        return money;
    }

    /**
     * Sets the amount of money the player has.
     *
     * @param playerMoney The new amount of money for the player.
     */
    public void setMoney(final int playerMoney) {
        this.money = playerMoney;
    }


    /**
     * Gets the current position of the player on the game board.
     *
     * @return The current position of the player.
     */
    public int getPosition() {
        return position;
    }

    /**
     * Sets the current position of the player on the game board.
     *
     * @param playerPosition The new position for the player.
     */
    public void setPosition(final int playerPosition) {
        this.position = playerPosition;
    }

    /**
     * Gets the players assets.
     *
     * @return the players assets.
     */
    public boolean getHasAssets() {
        return hasAssets;
    }

    /**
     * Set the players assets.
     *
     * @param assetsOwned
     */
    public void setHasAssets(final boolean assetsOwned) {
        this.hasAssets = assetsOwned;
    }




    /** List of all players in the game. */
    public static final ArrayList<Player> PLAYERSINTHEGAME = new ArrayList<>();

    /**
     * Gets the list of all players in the game.
     *
     * @return The list of all players.
     */
    public static ArrayList<Player> getPlayers() {
        return PLAYERSINTHEGAME;
    }

    /**
     * Handles bankruptcy of the player by selling assets if available, or removing the player from the game.
     *
     * @param player The player who is declaring bankruptcy.
     * @param board The game board containing property ownership information.
     */
    public void checkBankruptcy(final Player player, final Board board) {
        while (true) {
            if (player.getMoney() <= 0) {
                if (hasAssets) {
                    for (Integer propertyPosition : board.PROPERTY_OWNERSHIP.keySet()) {
                        if (board.PROPERTY_OWNERSHIP.get(propertyPosition)) {
                            board.PROPERTY_OWNERSHIP.put(propertyPosition, false);
                            player.addMoney(200);
                            System.out.println("You have sold " + board.getGameBoard(position) + " for £200 to survive bankruptcy");
                            System.out.println("You now have £" + player.getMoney() + " in your bank account");
                            player.setHasAssets(false);
                            return;
                        }
                    }
                }

                System.out.println("You have no assets to sell, therefore you are bankrupt!");
                System.out.println(player.getName() + " has been removed from the game");
                PLAYERSINTHEGAME.remove(player);
                break;

            } else {
                break;
            }
        }
    }
}
