package com.cm6123.monopoly;

import com.cm6123.monopoly.game.Board;
import com.cm6123.monopoly.game.GameAction;
import com.cm6123.monopoly.game.Player;
import org.junit.jupiter.api.Test;

import java.io.ByteArrayInputStream;

import static org.junit.jupiter.api.Assertions.*;

public class BankruptcyTest {

    @Test
    public void testBankruptcyIfNoAsset() {
        Board board = new Board();
        Player player1 = new Player("Player1", 10, 0); // Player1 starts with very low money (10)
        Player player2 = new Player("Player2", 1000, 0); // Player2 starts with sufficient money (1000)

        Player.getPlayers().add(player1);
        Player.getPlayers().add(player2);
        GameAction gameAction = new GameAction(board);

        // Land player 1 on tile 7 which is a train station to force player to go below 0
        player1.setPosition(7);
        ByteArrayInputStream in = new ByteArrayInputStream("R".getBytes());
        System.setIn(in);

        gameAction.landedOnTrainStation(player1);

        // Make player 1 now land on a property and type "N" to not buy the property
        player1.setPosition(9);
        in = new ByteArrayInputStream("N".getBytes());
        System.setIn(in);

        gameAction.landedOnProperty(player1);

        // Check bankruptcy and remove player if bankrupt
        if (player1.getMoney() <= 0) {
            player1.checkBankruptcy(player1, board);
            Player.getPlayers().remove(player1);
        }
        // checking if the player is bankrupt and removing the player
        // checking if there is only 1 player remaining since player 1 got removed
        assertTrue(player1.getMoney() <= 0);
        assertFalse(Player.getPlayers().contains(player1));
        assertEquals(1, Player.getPlayers().size());
        Player.getPlayers().remove(player1);
        Player.getPlayers().remove(player2);
    }


    @Test
    public void testBankruptcyIfPlayerHasAssets() {
        Board board = new Board();
        Player player1 = new Player("Player1", 210, 0);
        Player player2 = new Player("Player2", 1000, 0);

        Player.getPlayers().add(player1);
        Player.getPlayers().add(player2);


        GameAction gameAction = new GameAction(board);

        ByteArrayInputStream in = new ByteArrayInputStream("Y".getBytes());
        // Make the player land on a property and purchases the property, the player will have £10
        player1.setPosition(9);
        in = new ByteArrayInputStream("Y".getBytes());
        System.setIn(in);

        gameAction.landedOnProperty(player1);

        // Make player 1 land on a train station and pay the ticket so the players balance goes below 0
        player1.setPosition(7);
        in = new ByteArrayInputStream("R".getBytes());
        System.setIn(in);

        gameAction.landedOnTrainStation(player1);

        // Now make player 1 land on a property and type "N"

        if (player1.getMoney() <= 0 && player2.getHasAssets() == true) {
            // Checking if players money is below 0, if it is remove the player
            player1.checkBankruptcy(player1, board);
        }
        // The game should have forced sold the property the player had bought and added the funds to the players bank account
        // checking if the player got removed or not
        assertEquals(2, Player.getPlayers().size() );
        Player.getPlayers().remove(player1);
        Player.getPlayers().remove(player2);
        player1.setHasAssets(false);
    }
}

